/* GCompris - questions.js
 *
 * SPDX-FileCopyrightText: 2014 Arkit Vora <arkitvora123@gmail.com>
 *
 *   SPDX-License-Identifier: GPL-3.0-or-later
 */

function get() {
    return [
            [ // Level 1
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "A"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "B"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "C"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "D"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "E"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "F"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "G"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "H"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "I"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "J"
             }
            ],
            [ // Level 2
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "K"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "L"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "M"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "N"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "O"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "P"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "Q"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "R"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "S"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "T"
             }
            ],
            [ // Level 3
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "U"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "V"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "X"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "Y"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "Z"
             },
             {
                 "text": qsTr("Click on the dots in the braille cell area to produce the letter %1."),
                 "letter": "W"
             }
            ],
                [ // Level 4
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "1"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "2"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "3"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "4",
                     "braille_letter":[{"pos":1},{"pos":4},{"pos":5}]
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "5"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "6"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "7"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "8"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "9"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the number %1."),
                     "letter": "0"
                 }
                ],
                [ // Level 5
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the symbol %1."),
                     "letter": "+"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the symbol %1."),
                     "letter": "-"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the symbol %1."),
                     "letter": "*"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the symbol %1."),
                     "letter": "/"
                 },
                 {
                     "text": qsTr("Click on the dots in the braille cell area to produce the symbol %1."),
                     "letter": "#"
                 }
                ]
        ];
}
