/* GCompris - melodies.js
 *
 * SPDX-FileCopyrightText: 2016 Johnny Jazeix <jazeix@gmail.com>
 * SPDX-FileCopyrightText: 2018 Aman Kumar Gupta <gupta2140@gmail.com>
 *
 * Authors:
 *   Beth Hadley <bethmhadley@gmail.com> (GTK+ version)
 *   Johnny Jazeix <jazeix@gmail.com> (Qt Quick port)
 *   Aman Kumar Gupta <gupta2140@gmail.com> (Qt Quick port)
 *
 *   SPDX-License-Identifier: GPL-3.0-or-later
 **/

function get() {
    return [
             {
                 "title": "Twinkle Twinkle Little Star",
                 "_origin": qsTr("America: English Lullaby"),
                 "lyrics": "Twinkle, twinkle, little star, how I wonder what you are; Up above the world so high, like a diamond in the sky. Twinkle, twinkle, little star, how I wonder what you are.",
                 "defaultBPM": 120,
                 "melody": "Treble C4Quarter C4Quarter G4Quarter G4Quarter A4Quarter A4Quarter G4Half F4Quarter F4Quarter E4Quarter E4Quarter D4Quarter D4Quarter C4Half G4Quarter G4Quarter F4Quarter F4Quarter E4Quarter E4Quarter D4Half G4Quarter G4Quarter F4Quarter F4Quarter E4Quarter E4Quarter D4Half C4Quarter C4Quarter G4Quarter G4Quarter A4Quarter A4Quarter G4Half F4Quarter F4Quarter E4Quarter E4Quarter D4Quarter D4Quarter C4Whole"
             },
             {
                 "title": "Yankee Doodle",
                 "_origin": qsTr("America: Patriotic"),
                 "lyrics": "Yankee Doodle went to town, Riding on a pony; He stuck a feather in his hat, And called it macaroni",
                 "defaultBPM": 120,
                 "melody": "Treble G4Eighth G4Eighth A4Eighth B4Eighth G4Eighth B4Eighth A4Quarter G4Eighth G4Eighth A4Eighth B4Eighth G4Quarter F#4Quarter G4Eighth G4Eighth A4Eighth B4Eighth C5Eighth B4Eighth A4Eighth G4Eighth F#4Eighth D4Eighth E4Eighth F#4Eighth G4Quarter G4Quarter"
             },
             {
                 "title": "Simple Gifts",
                 "_origin": qsTr("America: Shaker Tune"),
                 "lyrics": "Tis the gift to be simple, 'tis the gift to be free, 'tis the gift to come down where we ought to be. And when we find ourselves in the place just right, 'Twill be in the valley of love and delight.",
                 "defaultBPM": 116,
                 "melody": "Treble C4Eighth C4Eighth F4Quarter F4Eighth G4Eighth A4Eighth F4Eighth A4Eighth Bb4Eighth C5Quarter C5Eighth Bb4Eighth A4Quarter G4Eighth F4Eighth G4Quarter C4Quarter G4Quarter F4Quarter G4Eighth A4Eighth G4Eighth E4Eighth C4Quarter C4Quarter F4Eighth E4Eighth F4Eighth G4Eighth A4Quarter G4Eighth G4Eighth A4Quarter Bb4Quarter C5Quarter A4Quarter G4Quarter G4Eighth G4Eighth A4Quarter A4Eighth G4Eighth F4Quarter F4Eighth F4Eighth F4Half"
             },
             {
                 "title": "Old MacDonald Had a Farm",
                 "_origin": qsTr("America: Nursery Rhyme"),
                 "lyrics": "Old MacDonald had a farm, EE-I-EE-I-O, And on that farm he had a [animal name], EE-I-EE-I-O",
                 "defaultBPM": 152,
                 "melody": "Bass G3Quarter G3Quarter G3Quarter D3Quarter E3Quarter E3Quarter D3Half B3Quarter B3Quarter A3Quarter A3Quarter G3Half D3Half G3Quarter G3Quarter G3Quarter D3Quarter E3Quarter E3Quarter D3Half B3Quarter B3Quarter A3Quarter A3Quarter G3Whole"
             },
             {
                 "title": "Un elefante se balanceaba",
                 "_origin": qsTr("Mexico"),
                 "lyrics": "Un elefante se columpiaba sobre la tela de una araña. Como veia que resistía fue a llamar a otro elefante",
                 "defaultBPM": 122,
                 "melody": "Treble G4Quarter G4Eighth G4Eighth E4Quarter E4Quarter G4Quarter G4Eighth G4Eighth E4Quarter E4Quarter G4Quarter G4Eighth G4Eighth A4Eighth G4Eighth F4Eighth E4Eighth F4Half D4Half F4Quarter F4Eighth F4Eighth D4Quarter D4Quarter F4Quarter F4Eighth F4Eighth D4Quarter D4Quarter F4Quarter F4Eighth F4Eighth G4Eighth F4Eighth E4Eighth D4Eighth E4Half C4Half"
             },
             {
                 "title": "Piove, Piove",
                 "_origin": qsTr("Italy"),
                 "lyrics": "Piove, piove. La gatta non si muove. La fiamma traballa. La mucca è nella stalla. La mucca ha il vitello. La pecora ha l’agnello. [La chioccia ha il pulcino.] Ognuno ha il suo bambino. Ognuno ha la sua mamma. E tutti fanno nanna!",
                 "defaultBPM": 116,
                 "melody": "Treble E4Eighth G4Quarter G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Eighth G4Eighth G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Eighth G4Eighth G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Eighth G4Eighth G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Half E4Eighth G4Half E4Eighth G4Eighth A4Eighth G4Eighth E4Eighth G4Half"
             },
             {
                 "title": "Que llueva, que llueva",
                 "_origin": qsTr("Spain"),
                 "lyrics": "Que llueva, que llueva. La Virgen de la Cueva. Los pajaritos cantan, Las nubes se levantan. ¡Que sí, que no, que caiga un chaparrón! Que siga lloviendo, Los pájaros corriendo. Florezca la pradera. Al sol de la primavera....",
                 "defaultBPM": 116,
                 "melody": "Treble E4Eighth G4Quarter G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Eighth G4Eighth G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Eighth G4Eighth G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Eighth G4Eighth G4Eighth A4Eighth G4Quarter G4Eighth E4Eighth G4Half E4Eighth G4Half E4Eighth G4Eighth A4Eighth G4Eighth E4Eighth G4Half"
             },
             {
                 "title": "Backe, backe Kuchen",
                 "_origin": qsTr("German Kid's Song"),
                 "lyrics": "Backe, backe, Kuchen Der Bäcker hat gerufen! Wer will guten Kuchen backen Der muß haben sieben Sachen: Eier und Schmalz Butter und Salz Milch und Mehl/Safran macht den Kuchen gehl!Schieb, schieb in´n Ofen ´nein.",
                 "defaultBPM": 92,
                 "melody": "Treble A4Quarter A4Quarter B4Quarter B4Quarter A4Half F#4Quarter D4Quarter A4Quarter A4Quarter B4Quarter B4Quarter A4Half F#4Half A4Quarter A4Quarter B4Quarter B4Quarter A4Quarter A4Quarter F#4Quarter D4Quarter A4Quarter A4Quarter B4Quarter B4Quarter A4Quarter A4Quarter F#4Quarter F#4Quarter A4Eighth A4Eighth A4Quarter F#4Half A4Eighth A4Eighth A4Quarter F#4Half A4Quarter A4Quarter F#4Half"
             },
             {
                 "title": "Se essa rua fosse minha",
                 "_origin": qsTr("Children's Song from Brazil"),
                 "lyrics": "Se essa rua Se essa rua fosse minha Eu mandava Eu mandava ladrilhar Com pedrinhas Com pedrinhas de brilhante Só pra ver Só pra ver meu bem passar Nessa rua Nessa rua tem um bosque Que se chama Que se chama solidão...",
                 "defaultBPM": 120,
                 "melody": "Treble E4Eighth E4Eighth A4Eighth A4Eighth E4Eighth C4Eighth A4Eighth C4Eighth F4Eighth E4Eighth E4Quarter D4Quarter E4Eighth E4Eighth B4Eighth B4Eighth F4Eighth E4Eighth G#4Eighth E4Eighth C4Eighth B4Eighth A4Half A4Eighth A4Eighth Bb4Eighth A4Eighth E4Eighth C#4Eighth A4Eighth G4Eighth F4Eighth E4Eighth G4Eighth F4Eighth E4Eighth D4Eighth E4Quarter B4Eighth G#4Eighth E4Eighth D4Eighth C4Eighth B4Eighth A4Half"
             },
             {
                 "title": "Fuchs du hast die Gans gestohlen",
                 "_origin": qsTr("Germany"),
                 "lyrics": "Fuchs, du hast die Gans gestohlen, gib sie wieder her, gib sie wieder her! Sonst wird dich der Jäger holen, mit dem Schießgewehr! Sonst wird dich der Jäger holen, mit dem Schießgewehr!",
                 "defaultBPM": 100,
                 "melody": "Treble D4Quarter E4Quarter F#4Quarter G4Quarter A4Quarter A4Quarter A4Quarter A4Quarter B4Quarter G4Quarter D5Quarter B4Quarter A4Whole B4Quarter G4Quarter D5Quarter B4Quarter A4Whole A4Quarter G4Quarter G4Quarter G4Quarter G4Quarter F#4Quarter F#4Quarter F#4Quarter F#4Quarter E4Quarter F#4Quarter E4Quarter D4Quarter F#4Quarter A4Half"
             },
             {
                 "title": "Un éléphant qui se balançait",
                 "_origin": qsTr("France"),
                 "lyrics": "Un éléphant qui se balançait\nSur une toile toile toile\nToile d'araignée\nIl trouvait ça tellement amusant\nQu'il alla chercher un\nDeuxième éléphant.",
                 "defaultBPM": 122,
                 "melody": "Treble G4Quarter G4Eighth G4Eighth E4Quarter E4Quarter G4Quarter G4Eighth G4Eighth E4Quarter E4Quarter G4Quarter G4Eighth G4Eighth A4Eighth G4Eighth F4Eighth E4Eighth F4Half D4Half F4Quarter F4Eighth F4Eighth D4Quarter D4Quarter F4Quarter F4Eighth F4Eighth D4Quarter D4Quarter F4Quarter F4Eighth F4Eighth G4Eighth F4Eighth E4Eighth D4Eighth E4Half C4Half"
             },
             {
                 "title": "Alle Meine Entchen",
                 "_origin": qsTr("Germany"),
                 "lyrics": "Alle meine Entchen schwimmen auf dem See, schwimmen auf dem See, Köpfchen in das Wasser, Schwänzchen in die Höh'.",
                 "defaultBPM": 100,
                 "melody": "Treble C4Eighth D4Eighth E4Eighth F4Eighth G4Quarter G4Quarter A4Eighth A4Eighth A4Eighth A4Eighth G4Half A4Eighth A4Eighth A4Eighth A4Eighth G4Half F4Eighth F4Eighth F4Eighth F4Eighth E4Quarter E4Quarter D4Eighth D4Eighth D4Eighth G4Eighth C4Half"
             },
             {
                 "title": "À la claire fontaine",
                 "_origin": qsTr("France"),
                 "lyrics": "À la claire fontaine\nM'en allant promener\nJ'ai trouvé l'eau si belle\nQue je m'y suis baigné\nIl y a longtemps que je t'aime\nJamais je ne t'oublierai.",
                 "defaultBPM": 77,
                 "melody": "Treble G4Quarter G4Eighth B4Eighth B4Eighth A4Eighth B4Eighth B4Eighth G4Quarter G4Eighth B4Eighth B4Eighth A4Eighth B4Quarter B4Quarter B4Eighth A4Eighth G4Eighth B4Eighth D5Eighth B4Eighth D5Quarter D5Eighth B4Eighth G4Eighth B4Eighth A4Half G4Half G4Quarter B4Quarter B4Quarter A4Eighth G4Eighth B4Quarter G4Quarter B4Half B4Quarter A4Eighth G4Eighth B4Quarter A4Quarter G4Whole"
             },
             {
                 "title": "O Cravo e a Rosa",
                 "_origin": qsTr("Brazil"),
                 "lyrics": "O Cravo brigou com a Rosa Debaixo de uma sacada O Cravo ficou ferido E a Rosa despedaçada O Cravo ficou doente A Rosa foi visitar O Cravo teve um desmaio A Rosa pôs-se a chorar",
                 "defaultBPM": 120,
                 "melody": "Treble G4Quarter G4Quarter E4Quarter C5Eighth B4Eighth A4Eighth G4Quarter F4Half A4Quarter A4Quarter F4Quarter C5Eighth B4Eighth A4Eighth G4Quarter G4Half G4Quarter C5Quarter C5Quarter C5Eighth D5Eighth C5Eighth B4Quarter A4Half A4Quarter G4Quarter B4Half A4Eighth F4Eighth D4Eighth C4Quarter"
             },
             {
                 "title": "Marcha Soldado",
                 "_origin": qsTr("Brazil"),
                 "lyrics": "Marcha soldado Cabeça de papel Se não marchar direito Vai preso pro quartel O quartel pegou fogo A polícia deu sinal Acode acode acode A bandeira nacional",
                 "defaultBPM": 120,
                 "melody": "Treble G4Quarter G4Eighth E4Eighth C4Quarter C4Eighth E4Eighth G4Eighth G4Eighth G4Eighth E4Eighth D4Half E4Eighth F4Eighth F4Eighth F4Eighth D4Eighth G4Quarter G4Eighth A4Eighth G4Eighth F4Eighth E4Eighth D4Eighth C4Half"
             },
             {
                 "title": "Frère jacques",
                 "_origin": qsTr("France"),
                 "lyrics": "Frère Jacques\nFrère Jacques\nDormez-vous ?\nDormez-vous ?\nSonnez les matines\nSonnez les matines\nDing ding dong\nDing ding dong",
                 "defaultBPM": 120,
                 "melody": "Treble F4Quarter G4Quarter A4Quarter F4Quarter F4Quarter G4Quarter A4Quarter F4Quarter A4Quarter Bb4Quarter C5Half A4Quarter Bb4Quarter C5Half C5Eighth D5Eighth C5Eighth Bb4Eighth A4Quarter F4Quarter C5Eighth D5Eighth C5Eighth Bb4Eighth A4Quarter F4Quarter F4Quarter C4Quarter F4Half F4Quarter C4Quarter F4Half"
             },
             {
                 "title": "Au clair de la lune",
                 "_origin": qsTr("France"),
                 "lyrics": "Au clair de la lune\nMon ami Pierrot\nPrête-moi ta plume\nPour écrire un mot\nMa chandelle est morte\nJe n'ai plus de feu\nOuvre-moi ta porte\nPour l'amour de Dieu.",
                 "defaultBPM": 85,
                 "melody": "Treble G4Quarter G4Quarter G4Quarter A4Quarter B4Half A4Half G4Quarter B4Quarter A4Quarter A4Quarter G4Whole G4Quarter G4Quarter G4Quarter A4Quarter B4Half A4Half G4Quarter B4Quarter A4Quarter A4Quarter G4Whole A4Quarter A4Quarter A4Quarter A4Quarter E4Half E4Half A4Quarter G4Quarter F#4Quarter E4Quarter D4Whole G4Quarter G4Quarter G4Quarter A4Quarter B4Half A4Half G4Quarter B4Quarter A4Quarter A4Quarter G4Whole"
             },
             {
                 "title": "Boci, boci tarka",
                 "_origin": qsTr("Hungary, Nursery Rhyme"),
                 "lyrics": "Boci, boci tarka, Se füle, se farka, Oda megyünk lakni, Ahol tejet kapni.",
                 "defaultBPM": 120,
                 "melody": "Treble D4Eighth F#4Eighth D4Eighth F#4Eighth A4Quarter A4Quarter D4Eighth F#4Eighth D4Eighth F#4Eighth A4Quarter A4Quarter D5Eighth C#5Eighth B4Eighth A4Eighth G4Quarter B4Quarter A4Eighth G4Eighth F#4Eighth E4Eighth D4Quarter D4Quarter"
             },
             {
                 "title": "Nyuszi ül a fűben",
                 "_origin": qsTr("Hungary, Nursery Rhyme"),
                 "lyrics": "Nyuszi ül a fűben, ülve szundikálva. Nyuszi talán beteg vagy, hogy már nem is ugorhatsz? Nyuszi hopp! Nyuszi hopp! Máris egyet elkapott.",
                 "defaultBPM": 120,
                 "melody": "Treble A4Eighth A4Eighth A4Eighth B4Eighth A4Quarter F#4Quarter A4Eighth A4Eighth A4Eighth B4Eighth A4Eighth G4Eighth F#4Quarter F#4Eighth E4Eighth F#4Eighth E4Eighth D4Eighth D4Eighth D4Quarter A4Eighth A4Eighth D5Quarter A4Eighth A4Eighth D5Quarter F#4Eighth E4Eighth F#4Eighth E4Eighth D4Eighth D4Eighth D4Quarter"
             },
             {
                 "title": "Lánc, lánc, eszterlánc",
                 "_origin": qsTr("Hungary, Children's Song"),
                 "lyrics": "Lánc, lánc, eszterlánc,/ eszterlánci cérna,/ cérna volna, selyem volna,/ mégis kifordulna. / Pénz volna karika, karika,/ forduljon ki Marika,/ Marikának lánca.",
                 "defaultBPM": 120,
                 "melody": "Treble A4Quarter B4Quarter A4Eighth A4Eighth F#4Quarter A4Eighth A4Eighth A4Eighth B4Eighth A4Quarter F#4Quarter A4Eighth A4Eighth A4Eighth B4Eighth A4Eighth G4Eighth F#4Eighth E4Eighth F#4Eighth E4Eighth F#4Eighth E4Eighth D4Quarter D4Quarter A4Quarter A4Eighth A4Eighth A4Eighth D5Eighth A4Quarter A4Eighth G4Eighth F#4Eighth E4Eighth D4Eighth D5Eighth A4Quarter A4Eighth G4Eighth F#4Eighth E4Eighth D4Quarter D4Quarter"
             },
             {
                 "title": "Tavaszi szél vizet áraszt",
                 "_origin": qsTr("Hungary, Children's Song"),
                 "lyrics": "Tavaszi szél vizet áraszt, virágom, virágom. / Minden madár társat választ virágom, virágom. / Hát én immár kit válasszak? Virágom, virágom. / Te engemet, én tégedet, virágom, virágom.",
                 "defaultBPM": 120,
                 "melody": "Treble F4Quarter G4Quarter A4Quarter A4Quarter G4Quarter G4Eighth A4Eighth F4Quarter G4Quarter A4Eighth A4Quarter G4Quarter G4Eighth A4Eighth F4Half C4Half F4Quarter G4Quarter A4Eighth A4Quarter G4Quarter G4Eighth A4Eighth F4Eighth E4Eighth D4Quarter G4Eighth G4Quarter A4Eighth F4Quarter E4Quarter D4Half D4Half"
             },
             {
                 "title": "Zec kopa repu",
                 "_origin": qsTr("Serbia"),
                 "lyrics": "Zec kopa repu, a lisica cveklu / Vuk im se prikrade, zeca da ukrade / Zec spazi repu, a lisica cveklu / Vuka nije bilo, u snu im se snilo",
                 "defaultBPM": 180,
                 "melody": "Treble A4Half A4Quarter F#4Quarter G4Half E4Half D4Quarter E4Quarter E4Quarter E4Quarter E4Half D4Half F4Quarter F4Quarter E4Quarter F4Quarter G4Half E4Half F4Quarter F4Quarter E4Quarter D4Quarter E4Half D4Half"
             },
             {
                 "title": "Jack and Jill",
                 "_origin": qsTr("Britain"),
                 "lyrics": "Jack and Jill went up the hill / To fetch a pail of water. / Jack fell down and broke his crown,/ And Jill came tumbling after",
                 "defaultBPM": 101,
                 "melody": "Treble C4Quarter C4Eighth D4Quarter D4Eighth E4Quarter E4Eighth F4Quarter F4Eighth G4Quarter G4Eighth A4Quarter A4Eighth B4Half C5Half C5Quarter C5Eighth B4Quarter B4Eighth A4Quarter A4Eighth G4Quarter G4Eighth F4Quarter F4Eighth E4Quarter E4Eighth D4Half C4Half"
             },
             {
                 "title": "Wlazł kotek na płotek",
                 "_origin": qsTr("Poland"),
                 "lyrics": "Wlazł kotek na płotek i mruga,/ Ładna to piosenka nie długa. / Nie długa, nie krótka, lecz w sam raz,/ Zaśpiewaj koteczku jeszcze raz.",
                 "defaultBPM": 120,
                 "melody": "Treble G4Quarter E4Quarter E4Quarter F4Quarter D4Quarter D4Quarter C4Eighth E4Eighth G4Half G4Quarter E4Quarter E4Quarter F4Quarter D4Quarter D4Quarter C4Eighth E4Eighth C4Half"
             },
             {
                 "title": "Φεγγαράκι μου λαμπρό",
                 "_origin": qsTr("Greece"),
                 "lyrics": "Φεγγαράκι μου λαμπρό,/ Φέγγε μου να περπατώ,/ Να πηγαίνω στο σχολειό / Να μαθαίνω γράμματα,/ Γράμματα σπουδάματα / Του Θεού τα πράματα.",
                 "defaultBPM": 120,
                 "melody": "Treble C4Quarter C4Quarter G4Quarter G4Quarter A4Quarter A4Quarter G4Half F4Quarter F4Quarter E4Quarter E4Quarter D4Quarter D4Quarter C4Half G4Quarter G4Quarter F4Quarter F4Quarter E4Quarter E4Quarter D4Half G4Quarter G4Quarter F4Quarter F4Quarter E4Quarter E4Quarter D4Half C4Quarter C4Quarter G4Quarter G4Quarter A4Quarter A4Quarter G4Half F4Quarter F4Quarter E4Quarter E4Quarter D4Quarter D4Quarter C4Whole"
             },
             {
                 "title": "Βρέχει, χιονίζει",
                 "_origin": qsTr("Greece"),
                 "lyrics": "Βρέχει, χιονίζει,/ Τα μάρμαρα ποτίζει,/ Κι' ο γέρος πα στα ξύλα / Κ' η γριά του μαγειρεύει / Κουρκούτι με το μέλι. / \"'Ελα γέρο μου να φας / Πού να φας την / κουτσουλιά,/ Και του σκύλλου την οριά.\"",
                 "defaultBPM": 120,
                 "melody": "Treble D4Eighth G4Eighth G4Eighth G4Eighth A4Eighth A4Eighth G4Eighth G4Eighth D4Eighth G4Eighth G4Eighth G4Eighth A4Eighth A4Eighth G4Eighth G4Eighth D4Eighth G4Eighth G4Eighth G4Eighth A4Eighth A4Eighth G4Eighth G4Eighth D4Eighth G4Eighth G4Eighth G4Eighth A4Eighth A4Eighth G4Eighth G4Eighth"
             },
             {
                 "title": "Котику сіренький",
                 "_origin": qsTr("Ukraine"),
                 "lyrics": "Котику сіренький,/ Котику біленький,/ Котку волохатий,/ Не ходи по хаті! / Не ходи по хаті,/ Не буди дитяти! / Дитя буде спати,/ Котик воркотати. / Ой на кота воркота,/ На дитину дрімота. / А-а, люлі!",
                 "defaultBPM": 120,
                 "melody": "Treble C4Quarter F4Quarter G4Quarter Bb4Eighth Ab4Eighth G4Quarter F4Quarter C4Quarter F4Quarter G4Quarter Bb4Eighth Ab4Eighth G4Half F4Half Bb4Quarter Bb4Quarter Bb4Quarter Ab4Quarter G4Half F4Half Bb4Quarter Bb4Quarter Bb4Quarter Ab4Quarter G4Half F4Half C4Quarter F4Quarter G4Quarter Bb4Eighth Ab4Eighth G4Eighth F4Eighth G4Quarter F4Half"
             },
             {
                 "title": "Baa, Baa, Blacksheep",
                 "_origin": qsTr("Britain"),
                 "lyrics": "Baa, baa, black sheep, / Have you any wool? / Yes, sir, yes, sir, / Three bags full; / One for the master, / And one for the dame, / And one for the little boy / Who lives down the lane",
                 "defaultBPM": 105,
                 "melody": "Treble G4Quarter G4Quarter D5Quarter D5Quarter E5Eighth E5Eighth E5Eighth E5Eighth D5Half C5Quarter C5Quarter B4Quarter B4Quarter A4Quarter A4Quarter G4Half D5Quarter D5Eighth D5Eighth C5Quarter C5Quarter B4Quarter B4Eighth B4Eighth A4Half D5Quarter D5Eighth D5Eighth C5Eighth C5Eighth C5Eighth C5Eighth B4Quarter B4Eighth B4Eighth A4Half"
             },
             {
                 "title": "A la rueda de San Miguel",
                 "_origin": qsTr("Mexico"),
                 "lyrics": "A la rueda, a la rueda de San Miguel / Todos traen su caja de miel. / A lo maduro, a lo maduro / Que se voltee (nombre del niño) de burro.",
                 "defaultBPM": 120,
                 "melody": "Treble G4Quarter G4Quarter C5Quarter G4Quarter C5Quarter G4Eighth G4Eighth C5Quarter D5Quarter E5Half C5Quarter D5Quarter E5Half C5Quarter C5Quarter D5Quarter D5Eighth D5Eighth G4Quarter G4Eighth G4Eighth C5Half"
             },
             {
                 "title": "Arroz con leche",
                 "_origin": qsTr("Mexico"),
                 "lyrics": "Arroz con leche / me quiero casar / con una mexicana / de la capital.",
                 "defaultBPM": 120,
                 "melody": "Treble C4Eighth F4Half A4Eighth F4Quarter F4Eighth C4Eighth F4Quarter F4Eighth A4Eighth G4Half A4Eighth Bb4Eighth A4Eighth G4Eighth F4Eighth E4Quarter E4Eighth C4Eighth D4Quarter E4Eighth E4Eighth F4Quarter"
             },
             {
                 "title": "Los pollitos dicen",
                 "_origin": qsTr("Mexico"),
                 "lyrics": "Los pollitos dicen / pío pío pío / cuando tienen hambre / cuando tienen frío",
                 "defaultBPM": 117,
                 "melody": "Treble A4Eighth E4Eighth A4Eighth B4Eighth C#5Quarter C#5Quarter B4Eighth D5Eighth C#5Eighth B4Eighth A4Quarter A4Quarter A4Eighth G#4Eighth F#4Eighth E4Eighth B4Quarter B4Quarter E4Eighth E4Eighth F#4Eighth G#4Eighth A4Quarter A4Quarter"
             },
             {
                 "title": "Dale dale dale",
                 "_origin": qsTr("Mexican song to break a piñata"),
                 "lyrics": "Dale dale dale / no pierdas el tino / porque si lo pierdes / pierdes el camino",
                 "defaultBPM": 120,
                 "melody": "Treble G4Eighth G4Eighth G4Eighth E4Eighth A4Quarter A4Quarter B4Eighth B4Eighth B4Eighth G4Eighth C5Quarter C5Quarter G4Eighth G4Eighth G4Eighth E4Eighth A4Quarter A4Quarter B4Eighth B4Eighth B4Eighth G4Eighth C5Half"
             },
             {
                 "title": "Kolme varista",
                 "_origin": qsTr("Finland"),
                 "lyrics": "Kolme varista istui aidalla. Silivati seilaa, silivati seilaa, yksi lensi pois. Kaksi varista istui aidalla. Silivati seilaa, silivati seilaa, yksi lensi pois. Yksi varis vain istui aidalla. Silivati seilaa, silivati seilaa, sekin lensi pois.",
                 "defaultBPM": 120,
                 "melody": "Treble A4Quarter Bb4Quarter A4Eighth F4Half D4Whole A4Quarter D4Quarter C5Quarter Bb4Quarter G4Whole A4Eighth A4Eighth A4Eighth G4Eighth E4Quarter G4Quarter A4Eighth A4Eighth A4Eighth G4Eighth E4Quarter G4Quarter A4Half G4Eighth F4Quarter E4Quarter D4Quarter"
             },
             {
                 "title": "Humpty Dumpty",
                 "_origin": qsTr("Britain"),
                 "lyrics": "Humpty Dumpty sat on a wall, / Humpty Dumpty had a great fall. / All the king's horses and all the king's men / Couldn't put Humpty together again",
                 "defaultBPM": 120,
                 "melody": "Treble E4Quarter G4Eighth F4Quarter A4Eighth G4Eighth A4Eighth B4Eighth C5Half E4Quarter G4Eighth F4Quarter A4Eighth G4Eighth F4Eighth E4Eighth D4Half E4Eighth E4Eighth G4Eighth F4Eighth F4Eighth A4Eighth G4Eighth A4Eighth B4Eighth C5Quarter C5Eighth E5Eighth E5Eighth C5Eighth F5Quarter E5Eighth D5Eighth C5Eighth B4Eighth C5Half"
             }
         ];
}
