/* GCompris - colors.js
 *
 * SPDX-FileCopyrightText: 2015 Bruno Coudoin
 *
 *   SPDX-License-Identifier: GPL-3.0-or-later
 */

function get() {
    return [
            [ // Level 1
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/yellow_duck.svg",
                 "text": qsTr("Find the yellow duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/yellow.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/black_duck.svg",
                 "text": qsTr("Find the black duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/black.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/green_duck.svg",
                 "text": qsTr("Find the green duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/green.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/red_duck.svg",
                 "text": qsTr("Find the red duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/red.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/white_duck.svg",
                 "text": qsTr("Find the white duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/white.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/blue_duck.svg",
                 "text": qsTr("Find the blue duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/blue.$CA")
             }
            ],
            [ // Level 2
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/yellow_duck.svg",
                 "text": qsTr("Find the yellow duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/yellow.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/black_duck.svg",
                 "text": qsTr("Find the black duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/black.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/brown_duck.svg",
                 "text": qsTr("Find the brown duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/brown.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/green_duck.svg",
                 "text": qsTr("Find the green duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/green.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/grey_duck.svg",
                 "text": qsTr("Find the grey duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/grey.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/orange_duck.svg",
                 "text": qsTr("Find the orange duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/orange.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/purple_duck.svg",
                 "text": qsTr("Find the purple duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/purple.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/white_duck.svg",
                 "text": qsTr("Find the white duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/white.$CA")
             }
            ],
            [ // Level 3
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/yellow_duck.svg",
                 "text": qsTr("Find the yellow duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/yellow.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/brown_duck.svg",
                 "text": qsTr("Find the brown duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/brown.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/green_duck.svg",
                 "text": qsTr("Find the green duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/green.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/grey_duck.svg",
                 "text": qsTr("Find the grey duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/grey.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/orange_duck.svg",
                 "text": qsTr("Find the orange duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/orange.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/purple_duck.svg",
                 "text": qsTr("Find the purple duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/purple.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/pink_duck.svg",
                 "text": qsTr("Find the pink duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/pink.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/blue_duck.svg",
                 "text": qsTr("Find the blue duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/blue.$CA")
             }
            ]
        ];
}
