/* GCompris - NoteNotations.js
 *
 * SPDX-FileCopyrightText: 2018 Aman Kumar Gupta <gupta2140@gmail.com>
 *
 * Authors:
 *   Beth Hadley <bethmhadley@gmail.com> (GTK+ version)
 *   Johnny Jazeix <jazeix@gmail.com> (Qt Quick port)
 *   Aman Kumar Gupta <gupta2140@gmail.com> (Qt Quick port)
 *
 *   SPDX-License-Identifier: GPL-3.0-or-later
 **/

function get() {
    return [
                {
                    "noteName": "TrebleD3",
                    "imageName": "trebleD3",
                    "rotation": 0,
                    "positionOnStaff": 9
                },
                {
                    "noteName": "TrebleE3",
                    "imageName": "trebleE3",
                    "rotation": 0,
                    "positionOnStaff": 8.5
                },
                {
                    "noteName": "TrebleF#3",
                    "imageName": "bassG4",
                    "rotation": 0,
                    "positionOnStaff": 8
                },
                {
                    "noteName": "TrebleF3",
                    "imageName": "bassG4",
                    "rotation": 0,
                    "positionOnStaff": 8
                },
                {
                    "noteName": "TrebleGb3",
                    "imageName": "bassF4",
                    "rotation": 0,
                    "positionOnStaff": 7.5
                },
                {
                    "noteName": "TrebleG3",
                    "imageName": "bassF4",
                    "rotation": 0,
                    "positionOnStaff": 7.5
                },
                {
                    "noteName": "TrebleG#3",
                    "imageName": "bassF4",
                    "rotation": 0,
                    "positionOnStaff": 7.5
                },
                {
                    "noteName": "TrebleA3",
                    "imageName": "trebleA3",
                    "rotation": 0,
                    "positionOnStaff": 7
                },
                {
                    "noteName": "TrebleA#3",
                    "imageName": "trebleA3",
                    "rotation": 0,
                    "positionOnStaff": 7
                },
                {
                    "noteName": "TrebleAb3",
                    "imageName": "trebleA3",
                    "rotation": 0,
                    "positionOnStaff": 7
                },
                {
                    "noteName": "TrebleB3",
                    "imageName": "trebleB3",
                    "rotation": 0,
                    "positionOnStaff": 6.5
                },
                {
                    "noteName": "TrebleBb3",
                    "imageName": "trebleB3",
                    "rotation": 0,
                    "positionOnStaff": 6.5
                },
                {
                    "noteName": "TrebleC4",
                    "imageName": "trebleC4",
                    "rotation": 0,
                    "positionOnStaff": 6
                },
                {
                    "noteName": "TrebleC#4",
                    "imageName": "trebleC4",
                    "rotation": 0,
                    "positionOnStaff": 6
                },
                {
                    "noteName": "TrebleDb4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 5.5
                },
                {
                    "noteName": "TrebleD4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 5.5
                },
                {
                    "noteName": "TrebleD#4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 5.5
                },
                {
                    "noteName": "TrebleEb4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 5
                },
                {
                    "noteName": "TrebleE4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 5
                },
                {
                    "noteName": "TrebleF4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 4.5
                },
                {
                    "noteName": "TrebleF#4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 4.5
                },
                {
                    "noteName": "TrebleG4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 4
                },
                {
                    "noteName": "TrebleGb4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 4
                },
                {
                    "noteName": "TrebleG#4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 4
                },
                {
                    "noteName": "TrebleAb4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 3.5
                },
                {
                    "noteName": "TrebleA4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 3.5
                },
                {
                    "noteName": "TrebleA#4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 3.5
                },
                {
                    "noteName": "TrebleBb4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 3
                },
                {
                    "noteName": "TrebleB4",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 3
                },
                {
                    "noteName": "TrebleC5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2.5
                },
                {
                    "noteName": "TrebleC#5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2.5
                },
                {
                    "noteName": "TrebleDb5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2
                },
                {
                    "noteName": "TrebleD5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2
                },
                {
                    "noteName": "TrebleD#5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2
                },
                {
                    "noteName": "TrebleEb5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1.5
                },
                {
                    "noteName": "TrebleE5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1.5
                },
                {
                    "noteName": "TrebleF5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1
                },
                {
                    "noteName": "TrebleF#5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1
                },
                {
                    "noteName": "TrebleG5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 0.5
                },
                {
                    "noteName": "TrebleGb5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 0.5
                },
                {
                    "noteName": "TrebleG#5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 0.5
                },
                {
                    "noteName": "TrebleAb5",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 0
                },
                {
                    "noteName": "TrebleA5",
                    "imageName": "trebleC4",
                    "rotation": 180,
                    "positionOnStaff": 0
                },
                {
                    "noteName": "TrebleA#5",
                    "imageName": "trebleC4",
                    "rotation": 180,
                    "positionOnStaff": 0
                },
                {
                    "noteName": "TrebleBb5",
                    "imageName": "trebleB3",
                    "rotation": 180,
                    "positionOnStaff": -0.5
                },
                {
                    "noteName": "TrebleB5",
                    "imageName": "trebleB3",
                    "rotation": 180,
                    "positionOnStaff": -0.5
                },
                {
                    "noteName": "TrebleC6",
                    "imageName": "trebleA3",
                    "rotation": 180,
                    "positionOnStaff": -1
                },
                {
                    "noteName": "TrebleC#6",
                    "imageName": "trebleA3",
                    "rotation": 180,
                    "positionOnStaff": -1
                },
                {
                    "noteName": "TrebleDb6",
                    "imageName": "bassF4",
                    "rotation": 180,
                    "positionOnStaff": -1.5
                },
                {
                    "noteName": "TrebleD6",
                    "imageName": "bassF4",
                    "rotation": 180,
                    "positionOnStaff": -1.5
                },
                {
                    "noteName": "TrebleD#6",
                    "imageName": "bassF4",
                    "rotation": 180,
                    "positionOnStaff": -1.5
                },
                {
                    "noteName": "TrebleEb6",
                    "imageName": "bassG4",
                    "rotation": 180,
                    "positionOnStaff": -2
                },
                {
                    "noteName": "TrebleE6",
                    "imageName": "bassG4",
                    "rotation": 180,
                    "positionOnStaff": -2
                },
                {
                    "noteName": "TrebleF6",
                    "imageName": "trebleE3",
                    "rotation": 180,
                    "positionOnStaff": -2.5
                },
                {
                    "noteName": "BassF1",
                    "imageName": "trebleD3",
                    "rotation": 0,
                    "positionOnStaff": 9
                },
                {
                    "noteName": "BassG1",
                    "imageName": "trebleE3",
                    "rotation": 0,
                    "positionOnStaff": 8.5
                },
                {
                    "noteName": "BassA1",
                    "imageName": "bassG4",
                    "rotation": 0,
                    "positionOnStaff": 8
                },
                {
                    "noteName": "BassB1",
                    "imageName": "bassF4",
                    "rotation": 0,
                    "positionOnStaff": 7.5
                },
                {
                    "noteName": "BassC2",
                    "imageName": "trebleA3",
                    "rotation": 0,
                    "positionOnStaff": 7
                },
                {
                    "noteName": "BassD2",
                    "imageName": "trebleB3",
                    "rotation": 0,
                    "positionOnStaff": 6.5
                },
                {
                    "noteName": "BassE2",
                    "imageName": "trebleC4",
                    "rotation": 0,
                    "positionOnStaff": 6
                },
                {
                    "noteName": "BassF2",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 5.5
                },
                {
                    "noteName": "BassG2",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 5
                },
                {
                    "noteName": "BassA2",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 4.5
                },
                {
                    "noteName": "BassB2",
                    "imageName": "genericNote",
                    "rotation": 0,
                    "positionOnStaff": 4
                },
                {
                    "noteName": "BassC3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 3.5
                },
                {
                    "noteName": "BassC#3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 3.5
                },
                {
                    "noteName": "BassDb3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 3
                },
                {
                    "noteName": "BassD3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 3
                },
                {
                    "noteName": "BassD#3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 3
                },
                {
                    "noteName": "BassEb3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2.5
                },
                {
                    "noteName": "BassE3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2.5
                },
                {
                    "noteName": "BassF3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2
                },
                {
                    "noteName": "BassF#3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 2
                },
                {
                    "noteName": "BassG3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1.5
                },
                {
                    "noteName": "BassGb3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1.5
                },
                {
                    "noteName": "BassG#3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1.5
                },
                {
                    "noteName": "BassAb3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1
                },
                {
                    "noteName": "BassA3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1
                },
                {
                    "noteName": "BassA#3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 1
                },
                {
                    "noteName": "BassBb3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 0.5
                },
                {
                    "noteName": "BassB3",
                    "imageName": "genericNote",
                    "rotation": 180,
                    "positionOnStaff": 0.5
                },
                {
                    "noteName": "BassC4",
                    "imageName": "trebleC4",
                    "rotation": 180,
                    "positionOnStaff": 0
                },
                {
                    "noteName": "BassC#4",
                    "imageName": "trebleC4",
                    "rotation": 180,
                    "positionOnStaff": 0
                },
                {
                    "noteName": "BassD4",
                    "imageName": "trebleB3",
                    "rotation": 180,
                    "positionOnStaff": -0.5
                },
                {
                    "noteName": "BassD#4",
                    "imageName": "trebleB3",
                    "rotation": 180,
                    "positionOnStaff": -0.5
                },
                {
                    "noteName": "BassDb4",
                    "imageName": "trebleB3",
                    "rotation": 180,
                    "positionOnStaff": -0.5
                },
                {
                    "noteName": "BassE4",
                    "imageName": "trebleA3",
                    "rotation": 180,
                    "positionOnStaff": -1
                },
                {
                    "noteName": "BassEb4",
                    "imageName": "trebleA3",
                    "rotation": 180,
                    "positionOnStaff": -1
                },
                {
                    "noteName": "BassF4",
                    "imageName": "bassF4",
                    "rotation": 180,
                    "positionOnStaff": -1.5
                },
                {
                    "noteName": "BassF#4",
                    "imageName": "bassF4",
                    "rotation": 180,
                    "positionOnStaff": -1.5
                },
                {
                    "noteName": "BassG4",
                    "imageName": "bassG4",
                    "rotation": 180,
                    "positionOnStaff": -2
                },
                {
                    "noteName": "BassGb4",
                    "imageName": "bassG4",
                    "rotation": 180,
                    "positionOnStaff": -2
                },
                {
                    "noteName": "BassA4",
                    "imageName": "trebleE3",
                    "rotation": 180,
                    "positionOnStaff": -2.5
                },
                {
                    "noteName": "BassB4",
                    "imageName": "trebleD3",
                    "rotation": 180,
                    "positionOnStaff": -3
                },
                {
                    "noteName": "wholeRest",
                    "imageName": "wholeRest",
                    "rotation": 0,
                    "positionOnStaff": 3.4
                },
                {
                    "noteName": "halfRest",
                    "imageName": "halfRest",
                    "rotation": 0,
                    "positionOnStaff": 3.4
                },
                {
                    "noteName": "quarterRest",
                    "imageName": "quarterRest",
                    "rotation": 0,
                    "positionOnStaff": 4.5
                },
                {
                    "noteName": "eighthRest",
                    "imageName": "eighthRest",
                    "rotation": 0,
                    "positionOnStaff": 4.6
                }
    ]
}
